@@ -1,4 +1,5 @@
 """Convert Markdown-it tokens to docutils nodes."""
+
 from __future__ import annotations
 
 import inspect
@@ -22,19 +23,18 @@ from typing import (
 )
 from urllib.parse import urlparse
 
+import docutils
 import jinja2
 import yaml
 from docutils import nodes
-from docutils.frontend import OptionParser
 from docutils.languages import get_language
-from docutils.parsers.rst import Directive, DirectiveError
+from docutils.parsers.rst import Directive, DirectiveError, directives, roles
 from docutils.parsers.rst import Parser as RSTParser
-from docutils.parsers.rst import directives, roles
 from docutils.parsers.rst.directives.misc import Include
 from docutils.parsers.rst.languages import get_language as get_language_rst
 from docutils.statemachine import StringList
 from docutils.transforms.components import Filter
-from docutils.utils import Reporter, new_document
+from docutils.utils import Reporter, SystemMessage, new_document
 from docutils.utils.code_analyzer import Lexer, LexerError, NumberLines
 from markdown_it import MarkdownIt
 from markdown_it.common.utils import escapeHtml
@@ -55,6 +55,7 @@ from myst_parser.mocking import (
 )
 from myst_parser.parsers.directives import MarkupError, parse_directive_text
 from myst_parser.warnings_ import MystWarnings, create_warning
+
 from .html_to_nodes import html_to_nodes
 
 if TYPE_CHECKING:
@@ -63,7 +64,14 @@ if TYPE_CHECKING:
 
 def make_document(source_path="notset", parser_cls=RSTParser) -> nodes.document:
     """Create a new docutils document, with the parser classes' default settings."""
-    settings = OptionParser(components=(parser_cls,)).get_default_values()
+    if docutils.__version_info__[:2] >= (0, 19):
+        from docutils.frontend import get_default_settings
+
+        settings = get_default_settings(parser_cls)
+    else:
+        from docutils.frontend import OptionParser
+
+        settings = OptionParser(components=(parser_cls,)).get_default_values()
     return new_document(source_path, settings=settings)
 
 
@@ -253,9 +261,9 @@ class DocutilsRenderer(RendererProtocol):
         # save for later reference resolution
         self.document.myst_slugs = self._heading_slugs
         if self._heading_slugs and self.sphinx_env:
-            self.sphinx_env.metadata[self.sphinx_env.docname][
-                "myst_slugs"
-            ] = self._heading_slugs
+            self.sphinx_env.metadata[self.sphinx_env.docname]["myst_slugs"] = (
+                self._heading_slugs
+            )
 
         # log warnings for duplicate reference definitions
         # "duplicate_refs": [{"href": "ijk", "label": "B", "map": [4, 5], "title": ""}],
@@ -372,7 +380,7 @@ class DocutilsRenderer(RendererProtocol):
     @contextmanager
     def current_node_context(
         self, node: nodes.Element, append: bool = False
-    ) -> Iterator:
+    ) -> Iterator[None]:
         """Context manager for temporarily setting the current node."""
         if append:
             self.current_node.append(node)
@@ -582,12 +590,15 @@ class DocutilsRenderer(RendererProtocol):
         self.add_line_and_source_path(quote, token)
         with self.current_node_context(quote, append=True):
             self.render_children(token)
-        if "attribution" in token.attrs:
-            attribution = nodes.attribution(
-                token.attrs["attribution"], "", nodes.Text(token.attrs["attribution"])
-            )
-            self.add_line_and_source_path(attribution, token)
-            quote.append(attribution)
+            if "attribution" in token.attrs:
+                attribution = nodes.attribution(token.attrs["attribution"], "")
+                self.add_line_and_source_path(attribution, token)
+                with self.current_node_context(attribution, append=True):
+                    self.nested_render_text(
+                        str(token.attrs["attribution"]),
+                        token_line(token, 0),
+                        inline=True,
+                    )
 
     def render_hr(self, token: SyntaxTreeNode) -> None:
         node = nodes.transition()
@@ -942,8 +953,12 @@ class DocutilsRenderer(RendererProtocol):
         """
         ref_node = nodes.reference()
         self.add_line_and_source_path(ref_node, token)
+        attribute_keys = ["class", "id", "reftitle", "target", "rel"]
+        if self.md_config.links_external_new_tab:
+            token.attrs["target"] = "_blank"
+            token.attrs["rel"] = "noreferer noopener"
         self.copy_attributes(
-            token, ref_node, ("class", "id", "reftitle"), aliases={"title": "reftitle"}
+            token, ref_node, attribute_keys, aliases={"title": "reftitle"}
         )
         uri = cast(str, token.attrGet("href") or "")
         implicit_text: str | None = None
@@ -1579,12 +1594,12 @@ class DocutilsRenderer(RendererProtocol):
                             from sphinx.domains.std import make_glossary_term
 
                             term = make_glossary_term(
-                                self.sphinx_env,  # type: ignore
+                                self.sphinx_env,  # type: ignore[arg-type]
                                 term.children,
-                                None,  # type: ignore
+                                None,  # type: ignore[arg-type]
                                 term.source,
                                 term.line,
-                                node_id=None,  # type: ignore
+                                node_id=None,
                                 document=self.document,
                             )
                         self.current_node.append(term)
@@ -1714,7 +1729,7 @@ class DocutilsRenderer(RendererProtocol):
         self.document.current_line = position
 
         # get directive class
-        output: tuple[Directive | None, list] = directives.directive(
+        output: tuple[Directive | None, list[SystemMessage]] = directives.directive(
             name, self.language_module_rst, self.document
         )
         directive_class, messages = output
@@ -1738,6 +1753,7 @@ class DocutilsRenderer(RendererProtocol):
                 directive_class,
                 first_line,
                 content,
+                line=position,
                 additional_options=additional_options,
             )
         except MarkupError as error:
@@ -1747,12 +1763,11 @@ class DocutilsRenderer(RendererProtocol):
             )
             return [error]
 
-        if parsed.warnings:
-            _errors = ",\n".join(parsed.warnings)
+        for _warning in parsed.warnings:
             self.create_warning(
-                f"{name!r}: {_errors}",
-                MystWarnings.DIRECTIVE_PARSING,
-                line=position,
+                f"{name!r}: {_warning.msg}",
+                _warning.type,
+                line=_warning.lineno if _warning.lineno is not None else position,
                 append_to=self.current_node,
             )
 
@@ -1799,9 +1814,7 @@ class DocutilsRenderer(RendererProtocol):
             result = [msg_node]
         except MockingError as exc:
             error_msg = self.reporter.error(
-                "Directive '{}' cannot be mocked: {}: {}".format(
-                    name, exc.__class__.__name__, exc
-                ),
+                f"Directive '{name}' cannot be mocked: {exc.__class__.__name__}: {exc}",
                 nodes.literal_block(content, content),
                 line=position,
             )
@@ -1813,9 +1826,7 @@ class DocutilsRenderer(RendererProtocol):
         for i in range(len(result)):
             assert isinstance(
                 result[i], nodes.Node
-            ), 'Directive "{}" returned non-Node object (index {}): {}'.format(
-                name, i, result[i]
-            )
+            ), f'Directive "{name}" returned non-Node object (index {i}): {result[i]}'
         return result
 
     def render_substitution_inline(self, token: SyntaxTreeNode) -> None:
@@ -1907,19 +1918,11 @@ def html_meta_to_nodes(
     if not data:
         return []
 
-    try:
-        meta_cls = nodes.meta
-    except AttributeError:
-        # docutils-0.17 or older
-        from docutils.parsers.rst.directives.html import MetaBody
-
-        meta_cls = MetaBody.meta
-
     output = []
 
     for key, value in data.items():
         content = str(value or "")
-        meta_node = meta_cls(content)
+        meta_node = nodes.meta(content)
         meta_node.source = document["source"]
         meta_node.line = line
         meta_node["content"] = content
